import { MediumAccount } from './medium.classes.account.js';
import { type IPostData, MediumPost } from './medium.classes.post.js';
import * as plugins from './medium.plugins.js';

export interface IMediumPublication {
  id: string;
  name: string;
  description: string;
  url: string;
  imageUrl: string;
}

export class MediumPublication implements IMediumPublication {
  // STATIC
  public static async getAllPublications(mediumAccount: MediumAccount) {
    await mediumAccount.readyDeferred.promise;
    const returnArray: MediumPublication[] = [];
    const response = await mediumAccount.request(`/users/${mediumAccount.id}/publications`, 'GET');
    const publicationsDataArray: IMediumPublication[] = response.body.data;
    for (const publicationData of publicationsDataArray) {
      const publication = new MediumPublication(mediumAccount, publicationData);
      returnArray.push(publication);
    }
    return returnArray;
  }

  public static async getOwnPublications(mediumAccount: MediumAccount) {
    await mediumAccount.readyDeferred.promise;
    const allPublications = await this.getAllPublications(mediumAccount);
    const ownPublications: MediumPublication[] = [];
    for (const publicationArg of allPublications) {
      const response = await mediumAccount.request(
        `/publications/${publicationArg.id}/contributors`,
        'GET'
      );
      const contributors: {
        publicationId: string;
        userId: string;
        role: string;
      }[] = response.body.data;
      for (const contributor of contributors) {
        if (contributor.userId === mediumAccount.id) {
          ownPublications.push(publicationArg);
          break;
        }
      }
    }
    return ownPublications;
  }

  public static async getPublicationByName(mediumAccountArg: MediumAccount, publicationNameArg: string) {
    const publications = await this.getAllPublications(mediumAccountArg);
    return publications.find(publicationArg => publicationArg.name === publicationNameArg);
  }

  // INSTANCE
  public mediumAccountRef: MediumAccount;

  id: string;
  name: string;
  description: string;
  url: string;
  imageUrl: string;

  constructor(mediumAccount: MediumAccount, dataArg: IMediumPublication) {
    this.mediumAccountRef = mediumAccount;
    Object.assign(this, dataArg);
  }

  public async createPost(dataArg: IPostData): Promise<MediumPost> {
    const result = await MediumPost.createPost(this, dataArg);
    return result;
  }
}
